body {
  text-align: center;
}
#result {
  padding-block: 10px;
}
div.badge {
  width: 50%;
  margin-left: auto;
  margin-right: auto;
}
table {
  padding-inline: 100px;
  caption-side: bottom;
  margin-left: auto;
  margin-right: auto;
}
th,
td {
  border: dashed black 1px;
  padding: 10px;
}
